@@ -21,6 +21,7 @@ use OCA\Bookmarks\Controller\Rest\TagsController;
 use OCA\Bookmarks\Controller\Rest\BookmarkController;
 use OCA\Bookmarks\Controller\Rest\InternalTagsController;
 use OCA\Bookmarks\Controller\Rest\InternalBookmarkController;
+use OCA\Bookmarks\Controller\Rest\PublicController;
 use OCP\IUser;
 
 class Application extends App {
@@ -51,10 +52,12 @@ class Application extends App {
 
 		$container->registerService('BookmarkController', function($c) {
 			/** @var IContainer $c */
+			$user = $c->query('ServerContainer')->getUserSession()->getUser();
+			$uid = is_null($user) ? null : $user->getUID();
 			return new BookmarkController(
 				$c->query('AppName'),
 				$c->query('Request'),
-				$c->query('ServerContainer')->getUserSession()->getUser()->getUID(),
+				$uid,
 				$c->query('ServerContainer')->getDatabaseConnection(),
 				$c->query('ServerContainer')->getL10NFactory()->get('bookmarks'),
 				$c->query('ServerContainer')->query(Bookmarks::class),
@@ -64,10 +67,12 @@ class Application extends App {
 		
 		$container->registerService('InternalBookmarkController', function($c) {
 			/** @var IContainer $c */
+			$user = $c->query('ServerContainer')->getUserSession()->getUser();
+			$uid = is_null($user) ? null : $user->getUID();
 			return new InternalBookmarkController(
 				$c->query('AppName'),
 				$c->query('Request'),
-				$c->query('ServerContainer')->getUserSession()->getUser()->getUID(),
+				$uid,
 				$c->query('ServerContainer')->getDatabaseConnection(),
 				$c->query('ServerContainer')->getL10NFactory()->get('bookmarks'),
 				$c->query('ServerContainer')->query(Bookmarks::class),
@@ -77,23 +82,40 @@ class Application extends App {
 
 		$container->registerService('TagsController', function($c) {
 			/** @var IContainer $c */
+			$user = $c->query('ServerContainer')->getUserSession()->getUser();
+			$uid = is_null($user) ? null : $user->getUID();
 			return new TagsController(
 				$c->query('AppName'),
 				$c->query('Request'),
-				$c->query('ServerContainer')->getUserSession()->getUser()->getUID(),
+				$uid,
 				$c->query('ServerContainer')->query(Bookmarks::class)
 			);
 		});
 		
 		$container->registerService('InternalTagsController', function($c) {
 			/** @var IContainer $c */
+			$user = $c->query('ServerContainer')->getUserSession()->getUser();
+			$uid = is_null($user) ? null : $user->getUID();
 			return new InternalTagsController(
 				$c->query('AppName'),
 				$c->query('Request'),
-				$c->query('ServerContainer')->getUserSession()->getUser()->getUID(),
+				$uid,
 				$c->query('ServerContainer')->query(Bookmarks::class)
 			);
 		});
+
+		$container->registerService('PublicController', function($c) {
+			/** @var IContainer $c */
+			$user = $c->query('ServerContainer')->getUserSession()->getUser();
+			$uid = is_null($user) ? null : $user->getUID();
+			return new PublicController(
+				$c->query('AppName'),
+				$c->query('Request'),
+				$uid,
+				$c->query('ServerContainer')->query(Bookmarks::class),
+				$c->query('ServerContainer')->getUserManager()
+			);
+		});
 	}
 
 }
