@@ -229,23 +229,39 @@ class BookmarkController extends ApiController {
 	 * @NoCSRFRequired
 	 * @CORS
 	 */
-	public function editBookmark($id = null, $url = "", $item = array(), $title = "", $is_public = false, $record_id = null, $description = "") {
+	public function editBookmark($id = null, $url = null, $item = null, $title = null, $is_public = null, $record_id = null, $description = null) {
 
-		// Check if it is a valid URL
-		$urlData = parse_url($url);
-		if(!$this->isProperURL($urlData)) {
+		if ($record_id !== null) {
+			$id = $record_id;
+		}
+		
+		$bookmark = $this->bookmarks->findUniqueBookmark($id, $this->userId);
+		$newProps = [
+			'url' => $url,
+			'title' => $title,
+			'is_public' => $is_public,
+			'description' => $description
+		];
+		if (is_array($item) && isset($item['tags']) && is_array($item['tags'])) {
+			$newProps['tags'] = $item['tags'];
+		}
+		foreach ($newProps as $prop => $value) {
+			if(!is_null($value)) {
+				$bookmark[$prop] = $value;
+			}
+		}
+
+		// Check if url and id are valid
+		$urlData = parse_url($bookmark['url']);
+		if(!$this->isProperURL($urlData) || !is_numeric($bookmark['id'])) {
 			return new JSONResponse(array(), Http::STATUS_BAD_REQUEST);
 		}
 
-		if ($record_id == null) {
-			return new JSONResponse(array(), Http::STATUS_BAD_REQUEST);
+		if ($bookmark['tags'] === false) {
+			$bookmark['tags'] = [];
 		}
 
-		$tags = isset($item['tags']) ? $item['tags'] : array();
-
-		if (is_numeric($record_id)) {
-			$id = $this->bookmarks->editBookmark($this->userId, $record_id, $url, $title, $tags, $description, $is_public = false);
-		}
+		$id = $this->bookmarks->editBookmark($this->userId, $bookmark['id'], $bookmark['url'], $bookmark['title'], $bookmark['tags'], $bookmark['description'], $bookmark['is_public']);
 
 		$bm = $this->bookmarks->findUniqueBookmark($id, $this->userId);
 		return new JSONResponse(array('item' => $bm, 'status' => 'success'));
@@ -289,22 +305,32 @@ class BookmarkController extends ApiController {
 	 * @NoAdminRequired
 	 */
 	public function clickBookmark($url = "") {
+		$url = urldecode($url);
 		$urlData = parse_url($url);
 		if(!$this->isProperURL($urlData)) {
-			return new JSONResponse(array(), Http::STATUS_BAD_REQUEST);
+			return new JSONResponse([], Http::STATUS_BAD_REQUEST);
+		}
+		// prepare url for query
+		$url = $this->db->escapeLikeParameter(htmlspecialchars_decode($url));
+
+		$config = \OC::$server->getConfig();
+		$escape = '';
+		if(
+			strpos($config->getSystemValue('dbtype'),'sqlite') !== false
+			&& version_compare($config->getSystemValue('version'), 12, '<')
+		) {
+			// sqlite requires the ESCAPE declaration, which is added per default as of Nc 12
+			$escape = ' ESCAPE \'\\\'';
 		}
 
-		$query = $this->db->prepareQuery('
-			UPDATE `*PREFIX*bookmarks`
-			SET `clickcount` = `clickcount` + 1
-			WHERE `user_id` = ?
-				AND `url` LIKE ?
-		');
+		$qb = $this->db->getQueryBuilder();
+		$qb->update('bookmarks')
+			->set('clickcount', $qb->createFunction('`clickcount` +1'))
+			->where($qb->expr()->eq('user_id', $qb->createNamedParameter($this->userId)))
+			->andWhere($qb->expr()->like('url', $qb->createNamedParameter($url)) . $escape)
+			->execute();
 
-		$params = array($this->userId, htmlspecialchars_decode($url));
-		$query->execute($params);
-
-		return new JSONResponse(array('status' => 'success'), Http::STATUS_OK);
+		return new JSONResponse(['status' => 'success'], Http::STATUS_OK);
 	}
 
 	/**
